'use strict';

var path = require('path');
var pkgDir = require('pkg-dir');
var fs = require('fs-extra');
var execa = require('execa');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var pkgDir__default = /*#__PURE__*/_interopDefaultLegacy(pkgDir);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var execa__default = /*#__PURE__*/_interopDefaultLegacy(execa);

const readPackageJson = async (startFrom = process.cwd()) => {
  const path$1 = await pkgDir__default["default"](startFrom);
  return fs__default["default"].readJSON(path.join(path$1, 'package.json'))
};

const mover = (src, dst) => fs__default["default"].move(src, dst, { overwrite: true });

var plugin = (userOptions = {}) => {
  const options = {
    outputDir: undefined, // where to put pack file
    rootDir: process.cwd(),
    packCommand: 'npm pack',
    packageJson: undefined, // permit testing without filesystem access
    mover, // permit testing without filesystem access
    shellCommand: (cmdString) => execa__default["default"].command(cmdString, { stdio: 'inherit' }), // permit testing without calling command
    ...userOptions
  };

  // Derive the pack file name from packageJson
  const packFileName = (packageJson) => {
    const name = packageJson.name.replace('@', '').replace('/', '-');
    const version = packageJson.version;
    return `${name}-${version}.tgz`
  };

  return {
    name: 'create-pack-file',
    // Create pack file, and move it to the output directory
    async generateBundle (outputOptions) {
      try {
        await options.shellCommand(options.packCommand);
        const packageJson = options.packageJson || await readPackageJson(options.rootDir);
        const packfile = packFileName(packageJson);
        const outputDir = options.outputDir || outputOptions.dir || path.dirname(outputOptions.file);
        await options.mover(path.join(options.rootDir, packfile), path.join(outputDir, packfile));
      } catch (error) {
        this.error(error);
      }
    }
  }
};

module.exports = plugin;
